//
//  ProjectionAppDelegate.m
//  Projection
//
//  Created by kunii on 09/05/26.
//  Copyright __MyCompanyName__ 2009. All rights reserved.
//

#import "ProjectionAppDelegate.h"

//	投影図描画用カスタムビュー定義
@interface ProjectionView : UIView {
}
@end

//	3次元座標用構造体
typedef struct {
	double	x;
	double	y;
	double	z;
} Point3d;

static double screen_z = 1.0;		//	目からモニタ画面までの距離
static double SCREEN_WIDE = 1.0;	//	モニタ画面3次元上での大きさ

//	透視変換。inPt3dの3次元座標を、モニタ画面上の2次元座標に変換してoutPt2dに入れて返す。
static void projection(const Point3d* inPt3d, CGPoint* outPt2d)
{
	double scale = screen_z / inPt3d->z;
	outPt2d->x = scale * inPt3d->x;
	outPt2d->y = scale * inPt3d->y;
}

//	3次元上の座標の平行移動
static void offset3d(Point3d* ioPt3d, const Point3d* inOffset)
{
	ioPt3d->x += inOffset->x;
	ioPt3d->y += inOffset->y;
	ioPt3d->z += inOffset->z;
}

//	3次元上のペンの移動
static void move3d(const Point3d* inPt3d)
{
	CGPoint pt;
	projection(inPt3d, &pt);
	CGContextRef context = UIGraphicsGetCurrentContext();
	CGContextMoveToPoint(context, pt.x, pt.y);
}

//	3次元上のラインの描画
static void line3d(const Point3d* inPt3d)
{
	CGPoint pt;
	projection(inPt3d, &pt);
	CGContextRef context = UIGraphicsGetCurrentContext();
	CGContextAddLineToPoint(context, pt.x, pt.y);
	CGContextStrokePath(context);
	CGContextMoveToPoint(context, pt.x, pt.y);
}

//	立方体の1面を描画
static void drawFrame(const Point3d* cube)
{
	const Point3d* firstpos = cube;
	move3d(cube++);
	for (int i = 0; i < 3; i++) {
		line3d(cube++);
	}
	line3d(firstpos);
}

//	与えられた座標を中心とする各辺の長さ1.0の立方体を1つ描画
static void drawCube(const Point3d* inOffset)
{
	/*
		[0]--------[1]
		|			|
		[3]--------[2]
	*/
	Point3d cube[] = {
		{-0.5,	0.5,	-0.5},
		{0.5,	0.5,	-0.5},
		{0.5,	-0.5,	-0.5},
		{-0.5,	-0.5,	-0.5},
		{-0.5,	0.5,	0.5},
		{0.5,	0.5,	0.5},
		{0.5,	-0.5,	0.5},
		{-0.5,	-0.5,	0.5}
	};
	
	for (int i = 0; i < 8; i++) {
		offset3d(&cube[i], inOffset);
	}
	drawFrame(&cube[0]);
	drawFrame(&cube[4]);
	for (int i = 0; i < 4; i++) {
		move3d(&cube[i]);
		line3d(&cube[i + 4]);
	}
}

//	投影図描画用カスタムビュー実装
@implementation ProjectionView

//	描画
- (void)drawRect:(CGRect)rect {
	CGContextRef context = UIGraphicsGetCurrentContext();
	CGContextSaveGState(context);
	//	3次元上のモニタ画面の大きさを1.0としてるので、そのままだと小さすぎる。画面全体にモニタ画面を割り当てるようCTMを調整。
	double width = self.frame.size.width;
	double height = self.frame.size.height;
	CGContextTranslateCTM(context, width / 2.0, height / 2.0);
	if (width < height)
		width = height;
	double scale = SCREEN_WIDE * width / 2.0;
	CGContextScaleCTM(context, scale, scale);
	CGContextSetLineWidth(context, 1.0 / scale);	//	CGContextScaleCTMによってラインの幅も影響を受けるので、その調整。
	
	CGContextSetRGBStrokeColor(context, 0.0, 1.0, 0.0, 1.0);	//	ライン色はグリーンあたりで...
	//	9つの立方体を描画。
	Point3d pos = {-1.1, -1.1, 3.0};
	for (int v = 0; v < 3; v++) {
		for (int h = 0; h < 3; h++) {
			drawCube(&pos);									//	立方体の描画
			pos.x += 1.1;
		}
		pos.x = -1.1;
		pos.y += 1.1;
	}
	
	CGContextRestoreGState(context);
}

@end


@implementation ProjectionAppDelegate

@synthesize window;

- (void)applicationDidFinishLaunching:(UIApplication *)application {    
	//	投影図描画用カスタムビューの作成と登録。
	UIView* view = [[ProjectionView alloc] initWithFrame:window.frame];
	[window addSubview:view];

    [window makeKeyAndVisible];
}


- (void)dealloc {
    [window release];
    [super dealloc];
}


@end
